<?php
namespace bwdrbgx_BookshelfGrid_namespace;

use bwdrbgx_BookshelfGrid_namespace\PageSettings\Page_Settings;
define( "bwdrbgx_ASFSK_ASSETS_PUBLIC_DIR_FILE", plugin_dir_url( __FILE__ ) . "assets/public" );
define( "bwdrbgx_ASFSK_ASSETS_ADMIN_DIR_FILE", plugin_dir_url( __FILE__ ) . "assets/admin" );
class Classbwdrbgx_BookshelfGrid {

	private static $_instance = null;

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function bwdrbgx_admin_editor_scripts() {
		add_filter( 'script_loader_tag', [ $this, 'bwdrbgx_admin_editor_scripts_as_a_module' ], 10, 2 );
	}

	public function bwdrbgx_admin_editor_scripts_as_a_module( $tag, $handle ) {
		if ( 'bwdrbgx_BookshelfGrid_editor' === $handle ) {
			$tag = str_replace( '<script', '<script type="module"', $tag );
		}

		return $tag;
	}

	private function include_widgets_files() {
		require_once( __DIR__ . '/widgets/bwdrbgx-bookshelf-grid-widgets.php' );
	}

	public function bwdrbgx_register_widgets() {
		// Its is now safe to include Widgets files
		$this->include_widgets_files();

		// Register WidgetsF
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\bwdrbgx_BookshelfGrid_widgets());
	}

	private function add_page_settings_controls() {
		require_once( __DIR__ . '/page-settings/bwdrbgx-bookshelf-grid-manager.php' );
		new Page_Settings();
	}

	// Register Category
	function bwdrbgx_BookshelfGrid_add_elementor_widget_categories( $elements_manager ) {

		$elements_manager->add_category(
			'bwdrbgx-BookshelfGrid-category',
			[
				'title' => esc_html__( 'Woocommerce Responsive Bookshelf Grid', 'bwdrbgx-woocommerce-bookshelf' ),
				'icon' => 'eicon-person',
			]
		);
	}

	//css-js-link-here
	public function bwdrbgx_all_assets_for_the_public(){
		$all_css_file = array(
			'bwdrbgx-BookshelfGrid-plugin-main' => array('bwdrbgx-BookshelfGrid_css_path_define'=>bwdrbgx_ASFSK_ASSETS_PUBLIC_DIR_FILE . '/css/main.css'),
        );
        foreach($all_css_file as $handle => $fileinfo){
            wp_enqueue_style( $handle, $fileinfo['bwdrbgx-BookshelfGrid_css_path_define'], null, '1.3', 'all');
        }

	}

	//admin-icon
	public function bwdrbgx_all_assets_for_elementor_editor_admin(){
		$all_css_js_file = array(
            'bwdrbgx-BookshelfGrid-admin-main' => array('bwdrbgx_path_admin_define'=>bwdrbgx_ASFSK_ASSETS_ADMIN_DIR_FILE . '/css/icon.css'),
        );
        foreach($all_css_js_file as $handle => $fileinfo){
            wp_enqueue_style( $handle, $fileinfo['bwdrbgx_path_admin_define'], null, '1.3', 'all');
        }
	}

	public function __construct() {

		// For public assets
		add_action('wp_enqueue_scripts', [$this, 'bwdrbgx_all_assets_for_the_public']);

		// For Elementor Editor
		add_action('elementor/editor/before_enqueue_scripts', [$this, 'bwdrbgx_all_assets_for_elementor_editor_admin']);

		// Register Category
		add_action( 'elementor/elements/categories_registered', [ $this, 'bwdrbgx_BookshelfGrid_add_elementor_widget_categories' ] );

		// Register widgets
		add_action( 'elementor/widgets/widgets_registered', [ $this, 'bwdrbgx_register_widgets' ] );

		// Register editor scripts
		add_action( 'elementor/editor/after_enqueue_scripts', [ $this, 'bwdrbgx_admin_editor_scripts' ] );

		$this->add_page_settings_controls();
	}
}

// Instantiate Plugin Class
Classbwdrbgx_BookshelfGrid::instance();