<?php
namespace bwdrbgx_BookshelfGrid_namespace\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Modules\DynamicTags\Module as TagsModule;


if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class bwdrbgx_BookshelfGrid_widgets extends Widget_Base {

    public function get_name() {
		return esc_html__( 'bwdrbgx-woocommerce-bookshelf-grid', 'bwdrbgx-woocommerce-bookshelf' );
	}

	public function get_title() {
		return esc_html__( 'Woocommerce Responsive Bookshelf Grid', 'bwdrbgx-woocommerce-bookshelf' );
	}

	public function get_icon() {
		return 'bwdrbgx-BookshelfGrid-icon eicon-woocommerce';
	}

	public function get_categories() {
		return [ 'bwdrbgx-BookshelfGrid-category' ];
	}

    // Function to fetch taxonomy terms dynamically
	protected function get_taxonomy_terms($taxonomy) {
        $terms = get_terms(array(
            'taxonomy' => $taxonomy,
            'hide_empty' => true,
        ));

        $options = array();
        if (!empty($terms) && !is_wp_error($terms)) {
            foreach ($terms as $term) {
                $options[$term->term_id] = $term->name;
            }
        }
        return $options;
    }

    protected function register_controls(){

		$this->start_controls_section(
			'bwdrbgx_query',
			[
				'label' => esc_html__( 'Product Filter', 'bwdrbgx-woocommerce-bookshelf' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

        $this->add_control(
			'bwdrbgx_custom_link',
			[
				'label' => esc_html__( 'Custom Link?', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'bwdrbgx-woocommerce-bookshelf' ),
				'label_off' => esc_html__( 'No', 'bwdrbgx-woocommerce-bookshelf' ),
				'return_value' => 'yes',
				'default' => 'no',
			]
		);
        $this->add_control(
			'bwdrbgx_custom_url',
			[
				'label' => esc_html__( 'Custom URL', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::URL,
				'placeholder' => esc_html__( 'https://example.com', 'bwdrbgx-woocommerce-bookshelf' ),
				'label_block' => true,
				'show_external' => true,
				'default' => [
                    'url' => 'https://bestwpdeveloper.com/',
                    'is_external' => true,
                    'nofollow' => true,
                ],
				'dynamic' => [
					'active' => true,
				],
				'condition' => [
                    'bwdrbgx_custom_link' => 'yes',
                ],
			]
		);

		$this->add_control(
            'stock_status',
            [
                'label'       => __( 'Select Stock Status', 'bwdrbgx-woocommerce-bookshelf' ),
                'type'        => Controls_Manager::SELECT,
                'options'     => [
                    'all'         => __( 'All', 'bwdrbgx-woocommerce-bookshelf' ),
                    'in_stock'    => __( 'In Stock', 'bwdrbgx-woocommerce-bookshelf' ),
                    'out_of_stock'=> __( 'Out Of Stock', 'bwdrbgx-woocommerce-bookshelf' ),
                ],
                'default'     => 'all',
            ]
        );

		$this->add_control(
            'selected_date',
            [
                'label'       => __( 'Select Date', 'bwdrbgx-woocommerce-bookshelf' ),
                'type'        => Controls_Manager::DATE_TIME,
                'description' => __( 'Filter products based on the selected date.', 'bwdrbgx-woocommerce-bookshelf' ),
                'default'     => '',
            ]
        );

		$this->add_control(
            'orderby',
            [
                'label'   => __( 'Order By', 'bwdrbgx-woocommerce-bookshelf' ),
                'type'    => Controls_Manager::SELECT,
                'options' => [
                    'date'       => __( 'Date', 'bwdrbgx-woocommerce-bookshelf' ),
                    'title'      => __( 'Title', 'bwdrbgx-woocommerce-bookshelf' ),
                    'rand'       => __( 'Random', 'bwdrbgx-woocommerce-bookshelf' ),
                    // Add more options as needed
                ],
                'default' => 'date',
            ]
        );

        $this->add_control(
            'order',
            [
                'label'   => __( 'Order', 'bwdrbgx-woocommerce-bookshelf' ),
                'type'    => Controls_Manager::SELECT,
                'options' => [
                    'ASC'  => __( 'Ascending', 'bwdrbgx-woocommerce-bookshelf' ),
                    'DESC' => __( 'Descending', 'bwdrbgx-woocommerce-bookshelf' ),
                ],
                'default' => 'DESC',
            ]
        );

		$category_options = $this->get_taxonomy_terms('product_cat');
		$category_options = ['' => __('All', 'bwdrbgx-woocommerce-bookshelf')] + $category_options;

		$tag_options = $this->get_taxonomy_terms('product_tag');
		$tag_options = ['' => __('All', 'bwdrbgx-woocommerce-bookshelf')] + $tag_options;

		$this->add_control(
			'selected_category',
			[
				'label'       => __( 'Select Category', 'bwdrbgx-woocommerce-bookshelf' ),
				'type'        => Controls_Manager::SELECT,
				'options'     => $category_options,
				'default'     => '',
				'description' => __( 'Filter products by the selected category.', 'bwdrbgx-woocommerce-bookshelf' ),
			]
		);

		$this->add_control(
			'selected_tag',
			[
				'label'       => __( 'Select Tag', 'bwdrbgx-woocommerce-bookshelf' ),
				'type'        => Controls_Manager::SELECT,
				'options'     => $tag_options,
				'default'     => '',
				'description' => __( 'Filter products by the selected tag.', 'bwdrbgx-woocommerce-bookshelf' ),
			]
		);

		$this->add_control(
            'included_products',
            [
                'label'       => __( 'Include Products', 'bwdrbgx-woocommerce-bookshelf' ),
                'type'        => Controls_Manager::TEXT,
                'description' => __( 'Enter product IDs to include (comma-separated).', 'bwdrbgx-woocommerce-bookshelf' ),
            ]
        );

        $this->add_control(
            'excluded_products',
            [
                'label'       => __( 'Exclude Products', 'bwdrbgx-woocommerce-bookshelf' ),
                'type'        => Controls_Manager::TEXT,
                'description' => __( 'Enter product IDs to exclude (comma-separated).', 'bwdrbgx-woocommerce-bookshelf' ),
            ]
        );
        $this->end_controls_section();

        $this->start_controls_section(
			'bwdrbgx_responsive_section',
			[
				'label' => esc_html__( 'Responsive Device', 'bwdrbgx-woocommerce-bookshelf' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

        $this->add_control(
			'bwdrbgx_important_note',
			[
				'label' => esc_html__( 'Important Note', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::RAW_HTML,
				'raw' => esc_html__( 'Request to you please do not change. It can be a problem to be responsive.', 'bwdrbgx-woocommerce-bookshelf' ),
				'content_classes' => 'bwdrbgx-note',
			]
		);

        $this->add_control(
			'bwdrbgx_row_countO',
			[
				'label' => esc_html__( 'Items In Row(600)', 'bwdrbgx-woocommerce-bookshelf' ),
                'description' => __( '@media (min-width:600px)', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 50,
				'step' => 1,
				'default' => 5,
			]
		);
        $this->add_control(
			'bwdrbgx_row_countT',
			[
				'label' => esc_html__( 'Items In Row(900)', 'bwdrbgx-woocommerce-bookshelf' ),
                'description' => __( '@media (min-width:900px)', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 50,
				'step' => 1,
				'default' => 7,
			]
		);
        $this->add_control(
			'bwdrbgx_row_countTH',
			[
				'label' => esc_html__( 'Items In Row(1200)', 'bwdrbgx-woocommerce-bookshelf' ),
                'description' => __( '@media (min-width:1200px)', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 50,
				'step' => 1,
				'default' => 9,
			]
		);
        $this->add_control(
			'bwdrbgx_row_countF',
			[
				'label' => esc_html__( 'Items In Row(1500)', 'bwdrbgx-woocommerce-bookshelf' ),
                'description' => __( '@media (min-width:1500px)', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 50,
				'step' => 1,
				'default' => 11,
			]
		);
        
        $this->add_control(
			'bwdrbgx_row_countFI',
			[
				'label' => esc_html__( 'Items In Row(1800)', 'bwdrbgx-woocommerce-bookshelf' ),
                'description' => __( '@media (min-width:1800px)', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 50,
				'step' => 1,
				'default' => 13,
			]
		);
        $this->add_control(
			'bwdrbgx_row_countS',
			[
				'label' => esc_html__( 'Items In Row(2100)', 'bwdrbgx-woocommerce-bookshelf' ),
                'description' => __( '@media (min-width:2100px)', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 1,
				'max' => 50,
				'step' => 1,
				'default' => 15,
			]
		);

		$this->end_controls_section();

        $this->start_controls_section(
			'bwdrbgx_style_section',
			[
				'label' => esc_html__( 'Background Color', 'bwdrbgx-woocommerce-bookshelf' ),
				'tab' => \Elementor\Controls_Manager::TAB_STYLE,
			]
		);

        $this->add_control(
			'bwdrbgx_image_width',
			[
				'label' => esc_html__( 'Image Width', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ '%', 'custom' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 1000,
						'step' => 5,
					],
					'%' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 43,
				],
				'selectors' => [
					'{{WRAPPER}} .bwdrbgx-container-wrapper img' => 'width: {{SIZE}}{{UNIT}}!important; height: auto!important;',
				],
			]
		);

        $this->add_control(
			'bwdrbgx_first_color',
			[
				'label' => esc_html__( 'First Background Color', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'default' => 'rgba(238, 188, 31, 1)'
			]
		);
        $this->add_control(
			'bwdrbgx_second_color',
			[
				'label' => esc_html__( 'Second Background Color', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'default' => 'rgba(255, 82, 145, 1)'
			]
		);
        $this->add_control(
			'bwdrbgx_third_color',
			[
				'label' => esc_html__( 'Third Background Color', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'default' => 'rgba(64, 98, 187, 1)'
			]
		);
        $this->add_control(
			'bwdrbgx_four_color',
			[
				'label' => esc_html__( 'Four Background Color', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'default' => 'rgba(6, 141, 126, 1)'
			]
		);
        $this->add_control(
			'bwdrbgx_five_color',
			[
				'label' => esc_html__( 'Five Background Color', 'bwdrbgx-woocommerce-bookshelf' ),
				'type' => \Elementor\Controls_Manager::COLOR,
				'default' => 'rgba(248, 255, 229, 1)'
			]
		);

		$this->end_controls_section();

		
	}

	protected function render(){
        $settings = $this->get_settings_for_display();
		$stock_status = isset($settings['stock_status']) ? $settings['stock_status'] : 'all';
		$selected_date = isset($settings['selected_date']) ? $settings['selected_date'] : '';
		$orderby = isset($settings['orderby']) ? $settings['orderby'] : 'date';
        $order = isset($settings['order']) ? $settings['order'] : 'DESC';
		$selected_category = isset($settings['selected_category']) ? $settings['selected_category'] : '';
        $selected_tag = isset($settings['selected_tag']) ? $settings['selected_tag'] : '';
		$included_products = isset($settings['included_products']) ? $settings['included_products'] : '';
        $excluded_products = isset($settings['excluded_products']) ? $settings['excluded_products'] : '';

        // Media Query
        $rows_countO = $settings['bwdrbgx_row_countO'];
        $rows_countT = $settings['bwdrbgx_row_countT'];
        $rows_countTH = $settings['bwdrbgx_row_countTH'];
        $rows_countF = $settings['bwdrbgx_row_countF'];
        $rows_countFI = $settings['bwdrbgx_row_countFI'];
        $rows_countS = $settings['bwdrbgx_row_countS']; ?>

        <style>
            @media (min-width:600px) {:root {--columns: <?php echo esc_attr($rows_countO); ?>!important;}}
            @media (min-width:900px) {:root {--columns: <?php echo esc_attr($rows_countT); ?>!important;}}
            @media (min-width:1200px) {:root {--columns: <?php echo esc_attr($rows_countTH); ?>!important;}}
            @media (min-width:1500px) {:root {--columns: <?php echo esc_attr($rows_countF); ?>!important;}}
            @media (min-width:1800px) {:root {--columns: <?php echo esc_attr($rows_countFI); ?>!important;}}
            @media (min-width:2100px) {:root {--columns: <?php echo esc_attr($rows_countS); ?>!important;}}

            :root {
                --yellow: <?php echo $settings['bwdrbgx_first_color'];?>;
                --pink: <?php echo $settings['bwdrbgx_second_color'];?>;
                --blue: <?php echo $settings['bwdrbgx_third_color'];?>;
                --green: <?php echo $settings['bwdrbgx_four_color'];?>;
                --white: <?php echo $settings['bwdrbgx_five_color'];?>;
                
            }
        </style>
        <?php
		$args = array(
            'post_type'      => 'product',
            'posts_per_page' => -1,
            'orderby'        => $orderby,
            'order'          => $order,
            // Add more arguments as needed for filtering or sorting the products
        );
		// Modify the query based on included products
        if (!empty($included_products)) {
            $args['post__in'] = array_map('intval', explode(',', $included_products));
        }
        // Modify the query based on excluded products
        if (!empty($excluded_products)) {
            $args['post__not_in'] = array_map('intval', explode(',', $excluded_products));
        }
        // Modify the query based on the chosen stock status
        if ($stock_status !== 'all') {
            $args['meta_query'] = array(
                array(
                    'key'     => '_stock_status',
                    'value'   => $stock_status === 'in_stock' ? 'instock' : 'outofstock',
                    'compare' => '=',
                ),
            );
        }
		// Modify the query to filter by the selected date
        if (!empty($selected_date)) {
            $args['date_query'] = array(
                array(
                    'after'     => date('Y-m-d', strtotime($selected_date)),
                    'inclusive' => true,
                ),
            );
        }
		// Modify the query based on the selected category
        if (!empty($selected_category)) {
            $args['tax_query'][] = array(
                'taxonomy' => 'product_cat',
                'field'    => 'term_id',
                'terms'    => $selected_category,
            );
        }
        // Modify the query based on the selected tag
        if (!empty($selected_tag)) {
            $args['tax_query'][] = array(
                'taxonomy' => 'product_tag',
                'field'    => 'term_id',
                'terms'    => $selected_tag,
            );
        }


		$products = new \WP_Query( $args );
		
		if ( $products->have_posts() ) {
			echo '<div class="bwdrbgx-container-wrapper">';
				echo '<div class="ul">';
				while ( $products->have_posts() ) {
					$products->the_post();
					?>
                        <div class="li">
                            <?php if( $settings["bwdrbgx_custom_link"] == 'yes' ){ ?>
                                <a href="<?php echo esc_url(isset($settings['bwdrbgx_custom_url']['url']) ? $settings['bwdrbgx_custom_url']['url'] : ''); ?>" <?php echo esc_attr(($settings['bwdrbgx_custom_url']['is_external']) ? 'target="_blank"' : ''); ?>>
                                    <?php the_post_thumbnail(); ?>
                                </a>
                            <?php } else{ ?>
                                <a href="<?php the_permalink(); ?>">
                                    <?php the_post_thumbnail(); ?>
                                </a>
                            <?php } ?>

						</div>

					<?php
				}
				echo '</div>';
			echo '</div>';
		} else {
			echo 'No products found';
		}
		wp_reset_postdata();

	}

}